library sma;

{$mode objfpc}{$H+}

type
  {MT4 stores the data of each candle in such a structure.
  MT4 will then give us a pointer to an array of such candles}
  TCandle = packed record
    time:   LongInt; // POSIX timestamp
    open,
    high,
    low,
    close,
    volume: Double;
  end;

  {the following would be a pretty large array but we actually never create
  an instance of it, we only need a typed pointer to such an array.
  We will later receive a pointer from MT4 to the chart history
  stored in such an array}
  THistory = array[0..MaxLongInt div SizeOf(TCandle) - 1] of TCandle;
  PHistory = ^THistory;

  {This is the structure of an indicator buffer. We will be given a
  pointer to an indicator buffer to fill it with the calculated values}
  TBuffer = array[0..MaxLongInt div SizeOf(Double) - 1] of Double;
  PBuffer = ^TBuffer;

const
  EMPTY_VALUE = $7FFFFFFF;


{calculate the moving average for the bar at index. Index is the absolute index
in the history array of the bar for which we want to calculate this value. The
function will return the value or it will return EMPTY_VALUE if there are not
enough bars before index to calculate a moving average of the requested period}
function movingAverage(history: PHistory; period: LongInt; index: LongInt): Double;
var
  sum: Double;
  i:   LongInt;

begin
  if index < period then exit(EMPTY_VALUE);
  if period < 1 then exit(history^[index].Close);

  sum := 0;
  for i := 0 to period - 1 do
  begin
    // index is the absolute index of the bar in the history array
    // for which to calculate the moving average. The UpdateBuffer
    // function has already transformed it for our convenience.
    sum += history^[index - i].Close;
  end;
  Result := sum / period;
end;

{Calculate for all (new) bars the indicator values and update the
indicator buffer. This is more or less the same thing that is found
in the example indicators from MetaQuotes. On every tick we look at
the numbers from Bars and IndicatorCounted() to determine over which
bars we have to loop and (re-)calculate the indicator value, only
here we do it in Pascal and it is nicely documented and the code
looks much cleaner}
procedure updateBuffer(history: PHistory; buffer: PBuffer; bars: LongInt; indicator_counted: LongInt; period: LongInt); stdcall;
var
  i:  LongInt;
  iC: LongInt;
  iB: LongInt;
begin
  // unlike in metatrader which silently tranforms the
  // indices of the buffers so that in mql4 the 0 is always
  // the last bar in the buffer, the real underlying arrays
  // with which we are dealing here directly start with 0
  // as the oldest bar and bars - 1 as the current bar.

  // We tranform bars and indicator_counted from MT4 into
  // abolute indices in our buffer arrays so that
  // iB will be the index of the current bar and
  // iC will be the index of the last seen bar
  // in most cases they will be identical and
  // we loop only one iteration: the current bar.
  // When a new bar has just appeared we will loop
  // over the last two bars.
  iC := indicator_counted - 1;
  iB := bars - 1;

  if indicator_counted = 0 then
  begin
    // this is a special case and happens only when it is called
    // for the very first time. We cannot start at index -1,
    // instead we start at 0 in this case.
    iC := 0;
  end;

  // loop over the new bars and calculate and update the
  // indicator buffer at these positions.
  for i := iC to iB do
  begin
    buffer^[i] := movingAverage(history, period, i);
  end;
end;

exports
  updateBuffer;

begin
end.

